//------------------------------------------------------------------------------
// MCL.js - Updates Menus with MCL Applications
// By Darrin W. Cullop (darrin.w.cullop@intel.com)
//
// INTEL CONFIDENTIAL
//	Copyright 2004-2005 Intel Corporation All Rights Reserved.  The source code
//	contained or described herein and all documents related to the source code
//	(Material) are owned by Intel Corporation or its suppliers or licensors.
//	Title to the Material remains with Intel Corporation or its suppliers and
//	licensors. The Material may contain trade secrets and 	proprietary and
//	confidential information of Intel Corporation and its 	suppliers and
//	licensors, and is protected by worldwide copyright and trade secret laws
//	and treaty provisions. No part of the Material may be used, copied,
//	reproduced, modified, published, uploaded, posted, transmitted, distributed,
//	or disclosed in any way without Intels prior express written permission. 
//	
//	No license under any patent, copyright, trade secret or other intellectual
//	property right is granted to or conferred upon you by disclosure or delivery
//	of the Materials, either expressly, by implication, inducement, estoppel or
//	otherwise. Any license under such intellectual property rights must be express
//	and approved by Intel in writing.
//
//	Unless otherwise agreed by Intel in writing, you may not remove or alter this
//	notice or any other notice embedded in Materials by Intel or Intels suppliers
//	or licensors in any way.
//
// Revision History:
//	0.90	In development
//	0.85	2004-12-25	Ecosystem Engineering Release
//------------------------------------------------------------------------------

var theTracker;

//------------------------------------------------------------------------------
// GetTracker - Gets the application tracker object
//------------------------------------------------------------------------------
function GetTracker()
{
	if ( theTracker == null )
	{
		try
		{
			var objCtrl = new ActiveXObject("MCLTrackerCtrl.MCLTrackerControl");
			theTracker = objCtrl.ApplicationTracker;
		}
		catch(e) { }
	}
	
	return theTracker;
}	

//------------------------------------------------------------------------------
// AddMclApps - Adds the entry points of a given category to a given grid
//				widget.
//
// Arguments:
//	objWidget	The grid widget that is going to contain the applications
//	strCategory The name of the category of applications to add
//------------------------------------------------------------------------------
function AddMclApps( objWidget, strCategory )
{
	var objTracker = GetTracker();
	if (!objTracker)
		return 0;

	var objAppList = objTracker.GetMCAppList();	
	var objEntryPoints = objAppList.GetEntryPoints( strCategory );
	
	// Check to see if there are any applications
	if ( objEntryPoints.count > 0 )
	{
		// Set the callback function
		objWidget.cbUpdate = UpdateMclGrid;
		
		// Add the name of the category as a property of the widget
		objWidget.strCategory = strCategory;
		
		// Initialize the Widget
		ScrollSetViewport( objWidget, 0, objEntryPoints.count );
	}
	else
	{
		// Hide the widget menu item
		WidgetSetVisible( objWidget, false );
	}
	
	return objEntryPoints.count;
}

//------------------------------------------------------------------------------
// UpdateMclGrid -	Callback function that actually adds the items to the grid
//------------------------------------------------------------------------------
function UpdateMclGrid( scroll, start, count )
{
	var objTracker = GetTracker();
	var objAppList = objTracker.GetMCAppList();	
	var objEntryPoints = objAppList.GetEntryPoints( scroll.strCategory );

	var row = 0, col = 0;
	var obj;
	var str;
	var strInner;
	
	for ( var i = start ; i < count ; i++ )
	{
		// Get the application entry point to add
		obj = objEntryPoints( scroll.viewFirst + i );
		
		// Set the thumbnail image
		strInner = "&nbsp;<br>";
		if (!(str = obj.ImageURL))
			str = "";
		strInner += '<img width="100" height="100" src="' + str + '"><br>';
		
		// Add the Title
		str = obj.Title;
		if ( str.length > 0 )
		{
			strInner += str;
		}
		
		strInner = '<div class="GridCell">'+strInner+'</div>';
		
		// Update the grid
		GridSetCell( scroll, row, col, strInner, MclAction, null, obj.URL );
		
		// Prepare for the next one
		if ( ++col >= scroll.navCols )
		{
			++row;
			col = 0;
		}
	}
}

//------------------------------------------------------------------------------
// MclAction -	Callback function that loads the corresponding application
//------------------------------------------------------------------------------
function MclAction( row, strURL )
{
	// Load the page
	PageLoad( strURL );
}
